/****************************************************************************
 Module
   EventCheckers.c

 Revision
   1.0.1 

 Description
   This is the sample for writing event checkers along with the event 
   checkers used in the basic framework test harness.

 Notes
   Note the use of static variables in sample event checker to detect
   ONLY transitions.
   
 History
 When           Who     What/Why
 -------------- ---     --------
 08/06/13 13:36 jec     initial version
****************************************************************************/

// this will pull in the symbolic definitions for events, which we will want
// to post in response to detecting events
#include "ES_Configure.h"
// this will get us the structure definition for events, which we will need
// in order to post events in response to detecting events
#include "ES_Events.h"
// if you want to use distribution lists then you need those function 
// definitions too.
#include "ES_PostList.h"
// This include will pull in all of the headers from the service modules
// providing the prototypes for all of the post functions
#include "ES_ServiceHeaders.h"
// this test harness for the framework references the serial routines that
// are defined in ES_Port.c
#include "ES_Port.h"
// include our own prototypes to insure consistency between header & 
// actual functionsdefinition
#include "EventCheckers.h"
#include "C32_PINDEFS.h"
#include "ADS12.h"
#include "SPI.h"

/*----------------------------- Module Defines ----------------------------*/
#define	KEYSTROKE_DEBUG_FLAG	1
#define	KEYSTROKE_DEBUG if (KEYSTROKE_DEBUG_FLAG) printf
#define	SPTEF_DEBUG_FLAG		0
#define	SPTEF_DEBUG if (SPTEF_DEBUG_FLAG) printf
#define	SPIF_DEBUG_FLAG		0
#define	SPIF_DEBUG if (SPIF_DEBUG_FLAG) printf
#define	BUTTON_DEBUG_FLAG		1
#define	BUTTON_DEBUG if (BUTTON_DEBUG_FLAG) printf

/****************************************************************************
 Function
   UpdateOrientationData
 Parameters
   None
 Returns
   bool: false
 Description
   Reads accelerometer outputs and updates orientation data.
 Notes

 Author
   BGG
****************************************************************************/
bool UpdateOrientationData(void)
{
  bool ReturnVal = false;

  if ( Calibrated() )
  {
      SetOrientationData();
  }
  
  return ReturnVal;
}


/****************************************************************************
 Function
   Check4ResetButtonPress
 Parameters
   None
 Returns
   bool: true if a new event was detected
 Description
   Checks for reset button press.
 Notes

 Author
   BGG
****************************************************************************/
bool Check4ResetButtonPress(void)
{
  static uint8_t LastResetButtonState = RESET_BUTTON; // starts HI
  uint8_t CurrentResetButtonState;
  bool ReturnVal = false;

  CurrentResetButtonState =  ( PTT & RESET_BUTTON );
  if ( (CurrentResetButtonState != LastResetButtonState) && (CurrentResetButtonState == 0) ) 
  {	
    ES_Event ThisEvent;
    ThisEvent.EventType = BUTTON_PRESSED;
    ThisEvent.EventParam = RESET;
    PostSPI_C32_FSM( ThisEvent );
    
    BUTTON_DEBUG("Reset button pressed.\n\r");
    ReturnVal = true;
  }
  LastResetButtonState = CurrentResetButtonState;

  return ReturnVal;
}

/****************************************************************************
 Function
   Check4TagInButtonPress
 Parameters
   None
 Returns
   bool: true if a new event was detected
 Description
   Checks for tag out button press.
 Notes

 Author
   BGG
****************************************************************************/
bool Check4TagInButtonPress(void)
{
  static uint8_t LastTagInButtonState = TAG_IN_BUTTON; // starts HI
  uint8_t CurrentTagInButtonState;
  uint8_t PlayerNumber, TeamColor;
  bool ReturnVal = false;

  CurrentTagInButtonState =  ( PTT & TAG_IN_BUTTON );
  if ( (CurrentTagInButtonState != LastTagInButtonState) && (CurrentTagInButtonState == 0) ) 
  {
  	ES_Event ThisEvent;
    ThisEvent.EventType = BUTTON_PRESSED;
    ThisEvent.EventParam = TAG_IN;
    PostSPI_C32_FSM( ThisEvent );
  	
  	 //PlayerNumber = (( PTT & PLAYER_SELECT_3 ) << 2 ) + (( PTT & PLAYER_SELECT_2 ) << 2 ) + (( PTM & PLAYER_SELECT_1 ) << 1 ) + (( PTM & PLAYER_SELECT_0 ) >> 1 );	 	 
    //TeamColor = ( PTT & TEAM_SELECT_TOGGLE );
    BUTTON_DEBUG("Tag in button pressed.\n\r"); // Player number = %x.", PlayerNumber);
    /*
    if ( TeamColor == 0 ) 
    {
    	printf(" Team color = RED.\n\r");
    } else if ( TeamColor == TEAM_SELECT_TOGGLE ) 
    {
    	printf(" Team color = GREEN.\n\r");
    }
    */
    ReturnVal = true;
  }
  LastTagInButtonState = CurrentTagInButtonState;

  return ReturnVal;
}

/****************************************************************************
 Function
   Check4TagOutButtonPress
 Parameters
   None
 Returns
   bool: true if a new event was detected
 Description
   Checks for tag out button press.
 Notes

 Author
   BGG
****************************************************************************/
bool Check4TagOutButtonPress(void)
{
  static uint8_t LastTagOutButtonState = TAG_OUT_BUTTON; // starts HI
  uint8_t CurrentTagOutButtonState;
  uint8_t TagOutLocation;
  bool ReturnVal = false;

  CurrentTagOutButtonState =  ( PTT & TAG_OUT_BUTTON );
  if ( (CurrentTagOutButtonState != LastTagOutButtonState) && (CurrentTagOutButtonState == 0) ) 
  {
  	ES_Event ThisEvent;
    ThisEvent.EventType = BUTTON_PRESSED;
    ThisEvent.EventParam = TAG_OUT;
    PostSPI_C32_FSM( ThisEvent );
  
  	TagOutLocation = ADS12_ReadADPin(0);	
    BUTTON_DEBUG("Tag out button pressed. Tag out location = %d\n\r", TagOutLocation);
    ReturnVal = true;
  }
  LastTagOutButtonState = CurrentTagOutButtonState;

  return ReturnVal;
}

/****************************************************************************
 Function
   Check4ReedSwitch
 Parameters
   None
 Returns
   bool: true if a new event was detected
 Description
   Checks for reed switch trigger.
 Notes

 Author
   BGG
****************************************************************************/
bool Check4ReedSwitch(void)
{
  static uint8_t LastReedSwitchState = REED_SWITCH; // starts HI
  uint8_t CurrentReedSwitchState;
  bool ReturnVal = false;

  CurrentReedSwitchState =  ( PTT & REED_SWITCH );
  if ( (CurrentReedSwitchState != LastReedSwitchState) && (CurrentReedSwitchState == 0) ) 
  {	
    ES_Event ThisEvent;
    ThisEvent.EventType = BUTTON_PRESSED;
    ThisEvent.EventParam = KICK;
    PostSPI_C32_FSM( ThisEvent );
    
    BUTTON_DEBUG("Reed switch triggered.\n\r");
    ReturnVal = true;
  }
  LastReedSwitchState = CurrentReedSwitchState;

  return ReturnVal;
}

/****************************************************************************
 Function
   Check4TransmitEmptyFlag
 Parameters
   None
 Returns
   bool: true if a new event was detected
 Description
   Posts an event if the transmit empty flag changes state AND is set.
 Notes

 Author
   BGG
****************************************************************************/
bool Check4TransmitEmptyFlag(void)
{
  static unsigned char LastTransmitEmptyFlagState = 0;
  unsigned char CurrentTransmitEmptyFlagState;
  bool ReturnVal = FALSE;

  CurrentTransmitEmptyFlagState =  (SPISR & _S12_SPTEF);
  
  // check that SPTEF flag is set AND has changed state since last check
  if ( (CurrentTransmitEmptyFlagState != LastTransmitEmptyFlagState) && (CurrentTransmitEmptyFlagState == _S12_SPTEF) )
  {                    
    ES_Event ThisEvent;
    ThisEvent.EventType = SPTEF_SET;
    PostSPI_C32_FSM( ThisEvent );
    ReturnVal = TRUE;
    SPTEF_DEBUG("SPTEF flag is set.\r\n");
  }
  LastTransmitEmptyFlagState = CurrentTransmitEmptyFlagState;

  return ReturnVal;
}

/****************************************************************************
 Function
   Check4ReceiveReadyFlag
 Parameters
   None
 Returns
   bool: true if a new event was detected
 Description
   Posts an event if the receive ready flag changes state AND is set.
 Notes

 Author
   BGG
****************************************************************************/
bool Check4ReceiveReadyFlag(void)
{
  static unsigned char LastReceiveReadyFlagState = 0;
  unsigned char CurrentReceiveReadyFlagState;
  bool ReturnVal = FALSE;

  CurrentReceiveReadyFlagState =  (SPISR & _S12_SPIF);
  
  // check that SPIF flag is set AND has changed state since last check
  if ( (CurrentReceiveReadyFlagState != LastReceiveReadyFlagState) &&
       (CurrentReceiveReadyFlagState == _S12_SPIF) )
  {                    
    ES_Event ThisEvent;
    ThisEvent.EventType = SPIF_SET;
    ThisEvent.EventParam = ReadByteSPI();
    PostSPI_C32_FSM( ThisEvent );
    ReturnVal = TRUE;
    SPIF_DEBUG("SPIF flag is set.\r\n");
  }
  LastReceiveReadyFlagState = CurrentReceiveReadyFlagState;

  return ReturnVal;
}

/****************************************************************************
 Function
   Check4Keystroke
 Parameters
   None
 Returns
   bool: true if a new key was detected & posted
 Description
   checks to see if a new key from the keyboard is detected and, if so, 
   retrieves the key and posts an ES_NewKey event to TestHarnessService1
 Notes
   The functions that actually check the serial hardware for characters
   and retrieve them are assumed to be in ES_Port.c
   Since we always retrieve the keystroke when we detect it, thus clearing the
   hardware flag that indicates that a new key is ready this event checker 
   will only generate events on the arrival of new characters, even though we
   do not internally keep track of the last keystroke that we retrieved.
 Author
   BGG
****************************************************************************/
bool Check4Keystroke(void)
{
  if ( IsNewKeyReady() ) // new key waiting?
  {
    ES_Event ThisEvent;
    ThisEvent.EventType = ES_NEW_KEY;
    ThisEvent.EventParam = GetNewKey();
    
    switch ( ThisEvent.EventParam )
    {
    	case 'p' :
    	{
    		KEYSTROKE_DEBUG("KEYSTROKE: Tag In button pressed.\n\r");
    		ThisEvent.EventType = BUTTON_PRESSED;
    		ThisEvent.EventParam = TAG_IN;
    		PostSPI_C32_FSM( ThisEvent ); 
    	}
    	break;
    	
    	case 't' :
    	{
    		KEYSTROKE_DEBUG("KEYSTROKE: Tag Out button pressed.\n\r");
    		ThisEvent.EventType = BUTTON_PRESSED;
    		ThisEvent.EventParam = TAG_OUT;
    		PostSPI_C32_FSM( ThisEvent ); 
    	}
    	break;
    	
    	case 'k' :
    	{
    		KEYSTROKE_DEBUG("KEYSTROKE: Kick triggered.\n\r");
    		ThisEvent.EventType = BUTTON_PRESSED;
    		ThisEvent.EventParam = KICK;
    		PostSPI_C32_FSM( ThisEvent ); 
    	}
    	break;
    	
    	case 'r' :
    	{
    		KEYSTROKE_DEBUG("KEYSTROKE: Reset button pressed.\n\r");
    		ThisEvent.EventType = BUTTON_PRESSED;
    		ThisEvent.EventParam = RESET;
    		PostSPI_C32_FSM( ThisEvent ); 
    	}
    	break;
    }
    return true;
  }
  return false;
}