/****************************************************************************
 Module
 	SPI.c

 Revision
	1.0.0

 Description
	SPI Module for the COACH C32.
   
 Notes
 
 ME 218c -- Team 13: Enchantment Under the Sea
 Author: Buddy Gardineer 
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
#include "ES_CheckEvents.h"
#include <stdio.h>
#include <hidef.h>			
#include <mc9s12c32.h>			    
#include <S12c32bits.h>	
#include <Bin_Const.h>		
#include <termio.h>					
#include "SPI.h"

/*----------------------------- Module Defines ----------------------------*/
#define	TX_DEBUG_FLAG	0
#define TX_DEBUG if (TX_DEBUG_FLAG) printf
#define	RX_DEBUG_FLAG	0
#define RX_DEBUG if (RX_DEBUG_FLAG) printf

/*----------------------------- Module Variables ----------------------------*/
static unsigned char RX_BYTE;

/*---------------------------- Module Functions ---------------------------*/


/****************************************************************************
 Function
 	InitSPI
 
 Description
 	Initializes the SPI Module for communication with the Xbee PIC and shift
 	register(s) on the COACH
 
 Inputs
 	None
 
 Returns
 	None
 
 Notes
 
 Author: Buddy Gardineer
 *****************************************************************************/
void InitSPI(void)
{
	puts("Initializing SPI...\r");

	// set SPI baud rate --> BaudRateDivisor = (SPPR+1)*2^(SPR+1)
	// for SPPR = 111 (7) and SPR = 111 (7), BaudRateDivisor = 2048
	// for 24MHz clock, baud rate is about 11719 bits/sec
	SPIBR = ( _S12_SPPR0 | _S12_SPPR1 | _S12_SPPR2 | _S12_SPR0 | _S12_SPR1 | _S12_SPR2 );

	// initialize SPI control register
	// CPOL = 1 (clock is active low)
	// CPHA = 1 (even edges)
	// MSTR = 1 (Master mode)
	// SPE = 1 (enable SPI)
	// NO SPI INTERRUPTS ENABLED
	SPICR1 = ( _S12_CPOL | _S12_CPHA | _S12_MSTR | _S12_SPE ); 

	// globally enable interrupts
	EnableInterrupts;

	puts("SPI initialized. \r\n");  
}

/****************************************************************************
 Function
 WriteByteSPI
 
 Description
 Writes a byte to the TX buffer.
 
 Inputs
 Unsigned char (one byte)
 
 Returns
 bool TRUE if byte was successfully loaded into TX buffer, FALSE otherwise
 
 Notes
 
 Author: Buddy Gardineer
 *****************************************************************************/
bool WriteByteSPI(unsigned char byte)
{
	bool returnVal = TRUE;
	
	// check that TX buffer is empty
    if ( (SPISR & _S12_SPTEF) != 0 )
    {
    	// write to TX buffer (this clears SPTEF)
        SPIDR = byte;
        TX_DEBUG("Transmitting %d...\r\n", byte);
    } else
    {
        // if TX buffer isn't empty, return FALSE
        puts("\n**** ERROR: TX BUFFER NOT EMPTY ****\r\n");
        returnVal = FALSE;
    }
    
    return returnVal;   
}

/****************************************************************************
 Function
 ReadByteSPI
 
 Description
 Reads a byte from the RX buffer.
 
 Inputs
 Unsigned char (one byte)
 
 Returns
 bool TRUE if byte was successfully loaded into TX buffer, FALSE otherwise
 
 Notes
 
 Author: Buddy Gardineer
 *****************************************************************************/
unsigned char ReadByteSPI(void)
{
	unsigned char returnVal = 0xFF;
	
	// check that RX buffer is ready to read
    if ( (SPISR & _S12_SPIF) != 0 )
    {
    	// read from RX buffer (this clears SPIF)
        returnVal = SPIDR;
        RX_DEBUG("BYTE READ.\r\n");
    } else
    {
        // if RX buffer isn't ready to read, return FALSE
        puts("\n**** ERROR: RX BUFFER NOT EMPTY ****\r\n");
    }
    
    return returnVal;   
}

/*----------------------- Interrupt Response Routines --------------------*/

// interrupt response is triggered when RX register is full (SPIF set)
  
void interrupt VectorNumber_Vspi SPI_INTERRUPT_RESPONSE(void)
{
    if( (SPISR & _S12_SPIF) != 0 )
    {
    	// read data in RX buffer (this clears SPIF)
    	RX_BYTE = SPIDR;
    }
}

