/****************************************************************************
 Module
   SPI_C32_FSM.c

 Revision
   1.0.0

 Description
   This is the FSM for SPI communication on the COACH C32. The C32 is in Master mode 
   and communicates with two Slaves: 1) one input/output shift register combo and 
   2) one PIC16F690 connected to the Xbee24 radio module.

 Notes

 ME 218c -- Team 13: Enchantment Under the Sea
 Author: Buddy Gardineer
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for the framework and this service
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_DeferRecall.h"
#include "SPI_C32_FSM.h"
#include "SPI.h"
#include "C32_PINDEFS.h"
#include <math.h>
#include "ADS12.h"

/*----------------------------- Module Defines ----------------------------*/
// these times assume a 1.024mS/tick timing
#define ONE_SEC 976
#define HALF_SEC (ONE_SEC/2)
#define TWO_SEC (ONE_SEC*2)
#define FIVE_SEC (ONE_SEC*5)

#define  PERIOD   		150
#define  KICK_PERIOD	   HALF_SEC

#define  EXTREME_ANGLE  (_M_PI/3)

#define  XBEE_XFER_DEBUG_FLAG	0
#define  XBEE_XFER_DEBUG if (XBEE_XFER_DEBUG_FLAG) printf
#define  SR_XFER_DEBUG_FLAG	0
#define  SR_XFER_DEBUG if (SR_XFER_DEBUG_FLAG) printf
#define  STATUS_BYTE_DEBUG_FLAG	0
#define  STATUS_BYTE_DEBUG if (STATUS_BYTE_DEBUG_FLAG) printf
#define  LED_BYTE_DEBUG_FLAG	0
#define  LED_BYTE_DEBUG if (LED_BYTE_DEBUG_FLAG) printf
#define  FSM_DEBUG_FLAG	0
#define  FSM_DEBUG if (FSM_DEBUG_FLAG) printf
#define	 TAG_IN_DEBUG_FLAG	0
#define	 TAG_IN_DEBUG if (TAG_IN_DEBUG_FLAG) printf
#define	 TAG_OUT_DEBUG_FLAG	1
#define	 TAG_OUT_DEBUG if (TAG_OUT_DEBUG_FLAG) printf
#define	 SPEED_DEBUG_FLAG	0
#define	 SPEED_DEBUG if (SPEED_DEBUG_FLAG) printf
#define	 ORIENTATION_DEBUG_FLAG	1
#define	 ORIENTATION_DEBUG if (ORIENTATION_DEBUG_FLAG) printf
#define	 KICK_DEBUG_FLAG	1
#define	 KICK_DEBUG if (KICK_DEBUG_FLAG) printf

#define SPEED			0
#define ORIENTATION		1

// header byte definitions
#define	SPEED_HEADER		0x00
#define	ORIENTATION_HEADER	0x01
#define TAG_IN_HEADER		0x02
#define	TAG_OUT_HEADER		0x03
#define	KICK_HEADER			0x04	
#define RESET_HEADER		0x05
#define KICK_CLEAR_HEADER	0x06

// STATUS_BYTE bits
#define	ACT		BIT7HI // HI if LiFKIM is currently active; LO otherwise
#define	PAIR	BIT6HI // HI if currently paired; LO otherwise
#define	LIFT1	BIT4HI // Lift Level bits
#define	LIFT2	BIT3HI
#define	ENR2	BIT2HI // Energy Store bits
#define	ENR1	BIT1HI
#define	ENR0	BIT0HI

#define PAIRED	(STATUS_BYTE & PAIR)

/*---------------------------- Module Functions ---------------------------*/
static void CalibrateAccel(void);
static void ParseStatusByte(void);
static void SaveNewTransferBytes(int ButtonPressed);
static void LoadTransferBytes(void);
static int8_t GetSpeedData(void);
static int8_t GetOrientationData(void);
static uint8_t GetTagOutData(void);
 
/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;
// add a deferral queue for up to 3 pending deferrals +1 to allow for ovehead
static ES_Event DeferralQueue[3+1];

static SPI_C32_FSM_t CurrentState;

static unsigned char HEADER_BYTE = 0x00;
static unsigned char DATA_BYTE = 0x00;

static unsigned char NEW_HEADER_BYTE = 0x00;
static unsigned char NEW_DATA_BYTE = 0x00;

static unsigned char LED_BYTE = 0x00; // contains pairing and energy store info to shift to LEDs
static unsigned char TEAM_PLAYER_BYTE; // contains team color and player number info
static unsigned char STATUS_BYTE; // contains pairing and energy store info from PLAYER

static float xCalib, yCalib, calibAngle; // accelerometer calibration values
static float xNeutral = 80, yNeutral = 90; // neutral x and y accel readings

static int8_t CurrentOrientation; 
static bool CALIBRATED = FALSE;
static bool NEW_TX_BYTES = FALSE;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitSPI_C32_FSM

 Parameters
     uint8_t : the priority of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, and does any 
     other required initialization for this service
 Notes

 Author
     Buddy Gardineer
****************************************************************************/
bool InitSPI_C32_FSM ( uint8_t Priority )
{
  ES_Event ThisEvent;

  MyPriority = Priority;
  
  // initializes SPI system
  InitSPI();
  // calibrate accelerometer
  CalibrateAccel();

  ES_InitDeferralQueueWith( DeferralQueue, ARRAY_SIZE(DeferralQueue) );
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService( MyPriority, ThisEvent) == true)
  {
      return true;
  }else
  {
      return false;
  }
}

/****************************************************************************
 Function
     PostSPI_C32_FSM

 Parameters
     EF_Event ThisEvent ,the event to post to the queue

 Returns
     bool false if the Enqueue operation failed, true otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     Buddy Gardineer
****************************************************************************/
bool PostSPI_C32_FSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunSPI_C32_FSM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
   
 Notes
   
 Author
   Buddy Gardineer
****************************************************************************/
ES_Event RunSPI_C32_FSM( ES_Event ThisEvent )
{
  ES_Event ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
  
  // if event is some sort of button press, need to update NEW_HEADER_BYTE and NEW_DATA_BYTE...
  if ( ThisEvent.EventType == BUTTON_PRESSED)
  {
      SaveNewTransferBytes( ThisEvent.EventParam );
      NEW_TX_BYTES = TRUE;
      FSM_DEBUG("BUTTON_PRESSED event received.\r\n");
  } else if ( (ThisEvent.EventType == ES_TIMEOUT)&&(ThisEvent.EventParam == KICK_CLEAR_TIMER) ) 
  {
	  NEW_HEADER_BYTE = KICK_CLEAR_HEADER;
	  NEW_DATA_BYTE = 0x00;
	  NEW_TX_BYTES = TRUE;
  	  KICK_DEBUG("KICK_CLEAR_TIMER expired.\n\r");	
  } else {
  
	  switch ( CurrentState )
	  {
	  	case InitPseudoState :
	  	{
	  	   FSM_DEBUG("__________InitPseudoState__________\r\n");
	  		if ( ThisEvent.EventType == ES_INIT )
	  		{
	  			ES_Timer_InitTimer(SPI_C32_FSM_TIMER, HALF_SEC);
	  			CurrentState = TransmittingHeaderByte;
	  		}
	  	}
	  	break;
	  	
	  	case TransmittingHeaderByte :
	  	{
	  	   FSM_DEBUG("______TransmittingHeaderByte_______\r\n");
	  		switch ( ThisEvent.EventType )
	  		{
	  			case ES_TIMEOUT :
	  			{
	            LoadTransferBytes();
	  				XBEE_XFER_DEBUG("--------START OF XBEE XFER--------- \n\n\r");
	  				XBEE_XFER_DEBUG("Transmitting header byte 0x%X...\n\r", HEADER_BYTE);
	  				// transmit the header byte
	  				PTM &= ~XBEE_PIC_SS; // lower Xbee PIC SS line
	  				if (HEADER_BYTE == TAG_IN_HEADER)
	  				{
	  				   printf("Sending tag in bytes...\n\r");
	  				}
	  				WriteByteSPI(HEADER_BYTE);
	  				CurrentState = TransmittingDataByte;
	  			}
	  			break;
	  		}
	  		
	  	}
	  	break;
	  	
	  	case TransmittingDataByte :
	  	{
	  	   FSM_DEBUG("_______TransmittingDataByte________\r\n");
	  		switch ( ThisEvent.EventType )
	  		{
	  			case SPIF_SET :
	  			{
	  				XBEE_XFER_DEBUG("Header byte transmitted.\n\n\r");
	  				XBEE_XFER_DEBUG("Transmitting data byte 0x%X...\n\r", DATA_BYTE);
	  				// we don't care about this byte's value
	  				// transmit data byte
	  				WriteByteSPI(DATA_BYTE);
	  				CurrentState = ParsingStatusByte;
	  			}
	  			break;
	  		}
	  		
	  	}
	  	break;
	  	
	  	case ParsingStatusByte :
	  	{
	  	   FSM_DEBUG("________ParsingStatusByte__________\r\n");
	  		switch ( ThisEvent.EventType )
	  		{
	  			case SPIF_SET :
	  			{
	  				XBEE_XFER_DEBUG("Data byte transmitted.\n\n\r");
	  				PTM |= XBEE_PIC_SS; // raise Xbee PIC SS line
	  				STATUS_BYTE = ThisEvent.EventParam;
	  				STATUS_BYTE_DEBUG("STATUS_BYTE = 0x%X\n\r", STATUS_BYTE);
	  				XBEE_XFER_DEBUG("---------END OF XBEE PIC XFER---------- \n\n\r");
	  				  				
	  				// process status byte
	  				ParseStatusByte();
	  				
	  				CurrentState = ShiftRegisterRW;
	  			    
	  			    // post event to this state machine to signal completion of Xbee PIC SPI sequence
	  		        ThisEvent.EventType = TWO_BYTE_TX_COMPLETE;
	  		        PostSPI_C32_FSM(ThisEvent);			
	  			}
	  			break;
	  		}
	  		
	  	}
	  	break;
	  	
	  	case ShiftRegisterRW :
	  	{
	  	   FSM_DEBUG("_________ShiftRegisterRW___________\r\n");
	  		switch ( ThisEvent.EventType )
	  		{
	  			case TWO_BYTE_TX_COMPLETE :
	  			{
	  				SR_XFER_DEBUG("---------START OF SR XFER---------- \n\n\r");
	  				PTAD &= ~SHIFT_REGISTER_SS; // lower shift register SS line
	  				
	  				LED_BYTE_DEBUG("Transmitting LED_BYTE 0x%X...\n\r", LED_BYTE);
	  				WriteByteSPI(LED_BYTE);
	  			}
	  			break;
	  			
	  			case SPIF_SET :
	  			{
	  				PTAD |= SHIFT_REGISTER_SS; // raise shift register SS line
	  				SR_XFER_DEBUG("LED_BYTE transmitted.\n\n\r");
	  				SR_XFER_DEBUG("---------END OF SR XFER---------- \n\n\r");
	  		      CurrentState = TransmittingHeaderByte;
	  		        
	  		      // start a timer; expiration will initiate another SPI cycle
	  		      ES_Timer_InitTimer(SPI_C32_FSM_TIMER, PERIOD);	
	  			}
	  			break;
	  		}
	  		
	  	}
	  	break;
	  	
	  }
  
  }

  return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/

static void CalibrateAccel(void)
{
	// read x and y accelerometer outputs and find initial orientation angle
	xCalib = xNeutral-(float)ADS12_ReadADPin(2);
	yCalib = yNeutral-(float)ADS12_ReadADPin(3);
	calibAngle = atan2f(yCalib, xCalib);

	ORIENTATION_DEBUG("xCalib = %f, ", xCalib);
	ORIENTATION_DEBUG("yCalib = %f\r\n", yCalib);
	ORIENTATION_DEBUG("calibAngle = %f\r\n", calibAngle*(180/_M_PI));
	
	CALIBRATED = TRUE; // set calibration flag
}

bool Calibrated(void) 
{
   return CALIBRATED;
}

static void ParseStatusByte(void) 
{
	unsigned char EnergyLevel; 
	
	if ( PAIRED )
	{
		// first, set pair LED bit
		LED_BYTE = BIT7HI;
		
		// extract a decimal value (1-7) for the energy level
		EnergyLevel = ( STATUS_BYTE & ( ENR2 | ENR1 | ENR0 ) );
		
		// shift bits to the correct locations for our hardware setup
		LED_BYTE |= ( 0x7F << (7-EnergyLevel) ) & BIT7LO;		
	}
	else
	{
		// if not paired, prepare to shut off all LEDs
		LED_BYTE = 0x00;
	} 	
}

static void SaveNewTransferBytes(int ButtonPressed)
{    
    uint8_t PlayerNumber, TeamColor;
    uint8_t PTM_COPY, PTT_COPY;
    switch ( ButtonPressed ) {
            
        case TAG_IN:
        {
            NEW_HEADER_BYTE = TAG_IN_HEADER;
            // load NEW_DATA_BYTE with player and team information
            PTM_COPY = PTM;
            PTT_COPY = PTT; 
            
            PlayerNumber = (( PTT_COPY & PLAYER_SELECT_3 ) << 2 ) + (( PTT_COPY & PLAYER_SELECT_2 ) << 2 ) + (( PTM_COPY & PLAYER_SELECT_1 ) << 1 ) + (( PTM_COPY & PLAYER_SELECT_0 ) >> 1 );	 	 
    	   	TeamColor = ( ( PTT_COPY & TEAM_SELECT_TOGGLE ) << 1 );
    	   	TEAM_PLAYER_BYTE = ( TeamColor | PlayerNumber );
    		
            TAG_IN_DEBUG("TEAM_PLAYER_BYTE = %x\n\r", TEAM_PLAYER_BYTE);
            // TODO: MAY NEED TO UPDATE
            NEW_DATA_BYTE = TEAM_PLAYER_BYTE;
        }
        break;
        
        case TAG_OUT:
        {
            NEW_HEADER_BYTE = TAG_OUT_HEADER;
            // load NEW_DATA_BYTE with tag out location information
            NEW_DATA_BYTE = GetTagOutData();
        }
        break;
        
        case KICK:
        {
            NEW_HEADER_BYTE = KICK_HEADER;
            // analog kick value for other 'bots? 0xFF for now...
            NEW_DATA_BYTE = 0x0F;
            ES_Timer_InitTimer(KICK_CLEAR_TIMER, KICK_PERIOD); // start kick clear timer
        }
        break;
        
        case RESET:
        {
            NEW_HEADER_BYTE = RESET_HEADER;
            // data byte doesn't matter for reset
            NEW_DATA_BYTE = 0xFF;
        }
        break;
    }
}

static void LoadTransferBytes(void)
{
    static unsigned char InfoToTransmit = SPEED;
    
    if ( NEW_TX_BYTES )
    {
    	// update transmit data consistent with most recent button press
        HEADER_BYTE = NEW_HEADER_BYTE;
        DATA_BYTE = NEW_DATA_BYTE;
        
        // clear the NEW_TX_BYTES flag
        NEW_TX_BYTES = FALSE;
    }
    else
    {
    	if ( InfoToTransmit == SPEED  )
    	{
    		HEADER_BYTE = SPEED_HEADER;
    		// fetch speed data
    		DATA_BYTE = GetSpeedData();
    	}
    	else if ( InfoToTransmit == ORIENTATION )
    	{
    		HEADER_BYTE = ORIENTATION_HEADER;
    		// fetch orientation data
    		SetOrientationData(); // read and parse orientation data 
    		DATA_BYTE = GetOrientationData();
    	}
    	// toggles between sending SPEED and ORIENTATION information
    	InfoToTransmit ^= 1;  
    }
}

static int8_t GetSpeedData(void)
{
	static int8_t LastSpeed = 0;
	int8_t CurrentSpeed;
	uint8_t FSRValue;
	FSRValue = ADS12_ReadADPin(5);
	
	if ( FSRValue < 100 ) // if reverse FSR is depressed
	{
	   // set speed to full reverse
	   CurrentSpeed = -128;
	}
	else
	{
   	// read current speed value
	   CurrentSpeed = ADS12_ReadADPin(1)/2; // read SoftPot input pin
	}

	SPEED_DEBUG("Current speed = %d\r\n", CurrentSpeed);
	
	LastSpeed = CurrentSpeed;
	return CurrentSpeed;
}

static int8_t GetOrientationData(void)
{
	ORIENTATION_DEBUG("Current orientation = %d\r\n", CurrentOrientation);
	return CurrentOrientation;
}

void SetOrientationData(void)
{
	static int8_t LastOrientation = 0;
	float xAcc, yAcc, angle;
	
	// read x and y accelerometer outputs and find orientation angle
	xAcc = xNeutral - (float)ADS12_ReadADPin(2);
	yAcc = yNeutral - (float)ADS12_ReadADPin(3);
	
	angle = calibAngle-atan2f(yAcc, xAcc);
	if ( angle > _M_PI )
	{
	   angle -= 2*_M_PI;
	}
	else if ( angle < -_M_PI )
	{
	   angle += 2*_M_PI;
	}

	// translate from angle (-pi to pi) to orientation values (-128 to 127)
	// use a cubic function with saturation to map values (less sensitive about zero,
	// more sensitive as you approach EXTREME_ANGLE, then constant beyond)
	if ( angle > EXTREME_ANGLE )
	{
	   CurrentOrientation = 127;
	}
	else if ( angle < -EXTREME_ANGLE )
	{
      CurrentOrientation = -128;
	}
	else
	{
	   CurrentOrientation = (int8_t) 127.0*pow(angle/EXTREME_ANGLE,3);   
   	// software low-pass
	   // CurrentOrientation = LastOrientation + 0.1*(CurrentOrientation - LastOrientation);
	}
	
	LastOrientation = CurrentOrientation;
}

static uint8_t GetTagOutData(void)
{
	uint8_t TagOutAnalog, TagOutLocation, TeamColor, TagOutData;
	
	// check tag out location
	TagOutAnalog = ADS12_ReadADPin(0); // read tag out location select input
	if ( TagOutAnalog < 20 )
	{
		TagOutLocation = 0;
	}
	else if ( (TagOutAnalog > 60) && (TagOutAnalog < 100) )
	{
		TagOutLocation = 1;
	}
	else if ( (TagOutAnalog > 150) && (TagOutAnalog < 190) )
	{
		TagOutLocation = 2;
	}
	else if ( TagOutAnalog > 240 )
	{
		TagOutLocation = 3;
	}
	else
	{
		printf("*****UNEXPECTED ANALOG VALUE DETECTED AT TAG OUT LOCATION SELECT INPUT*****");
	}
	
	TeamColor = ( PTT & TEAM_SELECT_TOGGLE );
	TagOutData = ( TagOutLocation << 1 ) + ( TeamColor >> 3 );
	TAG_OUT_DEBUG("TAG_OUT data byte = %x\r\n", TagOutData);
	 
	
	return TagOutData;
}


/*------------------------------- Footnotes -------------------------------*/
/*------------------------------ End of file ------------------------------*/

